# (c) 2014 Mind Candy Ltd. All Rights Reserved.
# Licensed under the MIT License; you may not use this file except in compliance with the License.
# You may obtain a copy of the License at http://opensource.org/licenses/MIT.

"""
Hooks that are added to lettuce to enable fattoush to work in the
manner described in the module's documentation. Just import this
module (`fattoush.hooks`) to apply them.
"""

from . import feature, scenario, step
