# This is done in order to apply said hooks, and may be
# changed in version 1.0 to avoid import-time behaviour
# as th8is can be confusing and results in misleading
# inspection messages such as 'Unused import statement'
#
# noinspection PyUnresolvedReferences
from fattoush import hooks
